#!/usr/bin/env python

"""
A convenience script engine to read vasp output in a directory tree.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Jul 9, 2012"


import argparse
import os
import re
import logging

from pymatgen.io.vaspio import Outcar
from pymatgen.util.string_utils import str_aligned
from pymatgen.apps.borg.hive import SimpleVaspToComputedEntryDrone, VaspToComputedEntryDrone
from pymatgen.apps.borg.queen import BorgQueen
import multiprocessing

save_file = 'vasp_data.gz'

def get_energies(rootdir, reanalyze, verbose, pretty, detailed, sort):
    if verbose:
        FORMAT = "%(relativeCreated)d msecs : %(message)s"
        logging.basicConfig(level=logging.INFO, format=FORMAT)

    if not detailed:
        drone = SimpleVaspToComputedEntryDrone(inc_structure=True)
    else:
        drone = VaspToComputedEntryDrone(inc_structure=True,
                                         data=['filename', 'initial_structure'])

    ncpus = multiprocessing.cpu_count()
    logging.info('Detected {} cpus'.format(ncpus))
    queen = BorgQueen(drone, number_of_drones=ncpus)
    if os.path.exists(save_file) and not reanalyze:
        msg = 'Using previously assimilated data from {}.'.format(save_file) + \
              ' Use -f to force re-analysis.'
        queen.load_data(save_file)
    else:
        queen.parallel_assimilate(rootdir)
        msg = 'Analysis results saved to {} for faster '.format(save_file) + \
              'subsequent loading.'
        queen.save_data(save_file)

    entries = queen.get_data()
    if sort == "energy_per_atom":
        entries = sorted(entries, key=lambda x:x.energy_per_atom)
    elif sort == "filename":
        entries = sorted(entries, key=lambda x:x.data['filename'])

    all_data = []
    for e in entries:
        if not detailed:
            delta_vol = "{:.2f}".format(e.data['delta_volume'] * 100)
        else:
            delta_vol = e.structure.volume / e.data['initial_structure'].volume - 1
            delta_vol = "{:.2f}".format(delta_vol * 100)
        all_data.append((e.data['filename'].replace("./", ""),
                     re.sub("\s+", "", e.composition.formula),
                     "{:.5f}".format(e.energy), "{:.5f}".format(e.energy_per_atom),
                     delta_vol))
    if len(all_data) > 0:
        headers = ("Directory", "Formula", "Energy", "E/Atom", "% vol chg")
        if pretty:
            from prettytable import PrettyTable
            t = PrettyTable(headers)
            t.set_field_align("Directory", "l")
            map(t.add_row, all_data)
            print t
        else:
            print str_aligned(all_data, headers)
        print msg
    else:
        print "No valid vasp run found."


def get_magnetizations(mydir, ion_list):
    data = []
    max_row = 0
    for (parent, subdirs, files) in os.walk(mydir):
        for f in files:
            if re.match("OUTCAR*", f):
                try:
                    row = []
                    fullpath = os.path.join(parent, f)
                    outcar = Outcar(fullpath)
                    mags = outcar.magnetization
                    mags = [m['tot'] for m in mags]
                    all_ions = xrange(len(mags))
                    row.append(fullpath.lstrip("./"))
                    if ion_list:
                        all_ions = ion_list
                    for ion in all_ions:
                        row.append(str(mags[ion]))
                    data.append(row)
                    if len(all_ions) > max_row:
                        max_row = len(all_ions)
                except:
                    pass

    for d in data:
        if len(d) < max_row + 1:
            d.extend([""] * (max_row + 1 - len(d)))
    headers = ["Filename"]
    for i in xrange(max_row):
        headers.append(str(i))
    print str_aligned(data, headers)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='''Convenient vasp run analyzer which can recursively go into a directory to search results.
    Author: Shyue Ping Ong
    Version: 1.0
    Last updated: Jul 14 2012''')
    parser.add_argument('directories', metavar='dir', default='.',
                        type=str, nargs='*', help='directory to process (default to .)')
    parser.add_argument('-e', '--energies', dest='get_energies',
                        action='store_true', help='Print energies')
    parser.add_argument('-m', '--mag', dest="ion_list", type=str, nargs=1,
                        help='Print magmoms. ION LIST can be a range (e.g., 1-2) or the string "All" for all ions.')
    parser.add_argument('-f', '--force', dest="reanalyze", action='store_true',
                        help='Force reanalysis. Typically, vasp_analyzer will just reuse a vasp_analyzer_data.gz if present. This forces the analyzer to reanalyze the data.')
    parser.add_argument('-v', '--verbose', dest="verbose", action='store_true',
                        help='verbose mode. Provides detailed output on progress.')
    parser.add_argument('-p', '--pretty', dest="pretty", action='store_const',
                        const=True, help='pretty mode. Uses prettytable to format output. Must have prettytable module installed.')
    parser.add_argument('-d', '--detailed', dest="detailed", action='store_true', help='Detailed mode. Parses vasprun.xml instead of separate vasp input. Slower.')
    parser.add_argument('-s', '--sort', dest="sort", type=str, nargs=1, default=['energy_per_atom'],
                        help='Sort criteria. Defaults to energy / atom.')

    args = parser.parse_args()

    default_energies = not (args.get_energies or args.ion_list)

    if args.get_energies or default_energies:
        for d in args.directories:
            get_energies(d, args.reanalyze, args.verbose, args.pretty,
                         args.detailed, args.sort[0])
    if args.ion_list:
        ion_list = list()
        if args.ion_list[0] == "All":
            ion_list = None
        else:
            (start, end) = map(int, re.split("-", args.ion_list[0]))
            ion_list = range(start, end + 1)
        for d in args.directories:
            get_magnetizations(d, ion_list)



#!/usr/bin/env python

from __future__ import division

'''
This module implements abstract base classes for post-processing entries.
Any class which modifies entries should inherit these classes.
'''

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Oct 6, 2011"

import abc

class EntryPostProcessor(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def process_entry(self, entry):
        """
        Process a single entry.
        
        Args:
            entry - An ComputedEntry object.
        
        Returns:
            An processed entry. None if entry is not compatible within the 
            processing scheme.
        """
        return

    @abc.abstractmethod
    def process_entries(self, entries):
        """
        Process a sequence of entries.
        
        Args:
            entries - A sequence of ComputedEntries.
        
        Returns:
            An list of processed entries.  ComputedEntries in the original list which 
            are not compatible with the processing scheme are excluded.
        """
        return

    @abc.abstractproperty
    def corrected_compound_formulas(self):
        """
        List of compound formulas that are corrected.
        """
        return



#!/usr/bin/env python

'''
Utilities for manipulating coordinates or list of coordinates
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Nov 27, 2011"

import numpy as np
import warnings
import math

from pymatgen.command_line.qhull_caller import qconvex


def in_coord_list(coord_list, coord, **kwargs):
    """
    Tests if a particular coord is within a coord_list using numpy.allclose.
    
    Args:
        coord_list:
            List of coords to test
        coord:
            Specific coordinates
        kwargs:
            keyword arguments supported by numpy.allclose. Please refer to 
            documentation for numpy.allclose.
    """
    for test in coord_list:
        if np.allclose(test, coord, **kwargs):
            return True
    return False


def get_linear_interpolated_value(x_values, y_values, x):
    """
    Returns an interpolated value by linear interpolation between two values.
    This method is written to avoid dependency on scipy, which causes issues on
    threading servers.
    
    Args:
        x_values:
            Sequence of x values.
        y_values:
            Corresponding sequence of y values
        x:
            Get value at particular x
    """
    val_dict = dict(zip(x_values, y_values))
    if x < min(x_values) or x > max(x_values):
        raise ValueError("x is out of range of provided x_values")

    for val in sorted(x_values):
        if val < x:
            x1 = val
        else:
            x2 = val
            break

    y1 = val_dict[x1]
    y2 = val_dict[x2]

    return y1 + (y2 - y1) / (x2 - x1) * (x - x1)


def get_convex_hull(coords, use_external_qhull=False):
    """
    Convenience method to compute convex hull for data using either scipy or
    an external call to qconvex.
    
    Args:
        coords:
            Sequence of sequence of floats, representing coordinates in N-D
            space.
        use_external_qhull:
            Set to True to force the use of the command line qhull.
            
    Returns:
        List of list of int, representing facets of the convex hull.
    """
    if not use_external_qhull:
        try:
            from scipy.spatial import Delaunay
            delau = Delaunay(coords)
            return delau.convex_hull
        except ImportError:
            warnings.warn("Error importing scipy.spatial. "
                          "Ignoring use_external_qhull = False and "
                          "attempting command-line qhull.")
    return qconvex(coords)


def coords_to_unit_cell(coords):
    """
    Map a set of fractional coordinates into the unit cell, i.e. 
    0 <= a < 1.
    
    Args:
        coords:
            A sequence of fractional coords.
    
    Returns:
        The coords mapped to within the unit cell.
    """
    return [i - math.floor(i) for i in coords]



#!/usr/bin/env python

'''
This module defines classes representing non-periodic and periodic sites.
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Jul 17, 2012"


import math
import collections
import itertools

import numpy as np
from pymatgen.core.lattice import Lattice
from pymatgen.core.periodic_table import Element, Specie, smart_element_or_specie
from pymatgen.serializers.json_coders import MSONable
from pymatgen.util.coord_utils import coords_to_unit_cell

class Site(collections.Mapping, collections.Hashable, MSONable):
    '''
    A generalized *non-periodic* site. Atoms and occupancies should be a dict
    of element:occupancy or an element, in which case the occupancy default to 
    1. Coords are given in standard cartesian coordinates.
    '''

    supported_properties = ('magmom', 'charge', 'coordination_no', 'forces')

    def __init__(self, atoms_n_occu, coords, properties=None):
        """
        Create a *non-periodic* site.
        
        Args:
            atoms_n_occu: 
                Species on the site. Can be:
                
                i.  A sequence of element / specie specified either as string
                    symbols, e.g. ["Li", "Fe2+", "P", ...] or atomic numbers,
                    e.g., (3, 56, ...) or actual Element or Specie objects.
                
                ii. List of dict of elements/species and occupancies, e.g.,
                    [{'Fe' : 0.5, 'Mn':0.5}, ...]. This allows the setup of
                    disordered structures.
            coords: 
                Cartesian coordinates of site.
            properties:
                Properties associated with the site as a dict, e.g. 
                {'magmom':5}. Defaults to None.
        """

        if isinstance(atoms_n_occu, dict):
            self._species = {smart_element_or_specie(k): v \
                             for k, v in atoms_n_occu.items()}
            totaloccu = sum(self._species.values())
            if totaloccu > 1:
                raise ValueError("Species occupancies sum to more than 1!")
            self._is_ordered = (totaloccu == 1 and len(self._species) == 1)
        else:
            self._species = {smart_element_or_specie(atoms_n_occu):1}
            self._is_ordered = True
        self._coords = coords
        self._properties = properties if properties else {}
        for k in self._properties.keys():
            if k not in Site.supported_properties:
                raise ValueError("{} is not a supported property".format(k))

    @property
    def properties(self):
        """
        Returns a view of properties as a dict.
        """
        return {k:v for k, v in self._properties.items()}

    def __getattr__(self, a):
        if a in self._properties:
            return self._properties[a]
        raise AttributeError(a)

    def distance(self, other):
        """
        Get distance between two sites.
        
        Args:
            other:
                Other site.
        """
        return np.linalg.norm(other.coords - self.coords)

    def distance_from_point(self, pt):
        """
        Returns distance between the site and a point in space.
        
        Args:
            pt:
                cartesian coordinates of point.
        """
        return np.linalg.norm(np.array(pt) - np.array(self._coords))

    @property
    def species_string(self):
        """
        String representation of species on the site.
        """
        if self._is_ordered:
            return str(self._species.keys()[0])
        else:
            sorted_species = sorted(self._species.keys())
            return ', '.join(["%s:%.3f" % (str(sp), self._species[sp]) \
                              for sp in sorted_species])

    @property
    def species_and_occu(self):
        """
        The species at the site, i.e., a dict of element and occupancy
        """
        return self._species.copy()

    @property
    def specie(self):
        """
        .. deprecated:: 1.0
            Use :func:`species_and_occu` instead.
            
        The Specie/Element at the site. Only works for ordered sites. Otherwise 
        an AttributeError is raised. Use this property sparingly.  Robust
        design should make use of the property species_and_occu instead.
        
        Raises:
            AttributeError if Site is not ordered.
        """
        if not self._is_ordered:
            raise AttributeError("specie property only works for ordered sites!")
        return self._species.keys()[0]

    @property
    def coords(self):
        """
        A copy of the cartesian coordinates of the site as a numpy array.
        """
        return np.copy(self._coords)

    @property
    def is_ordered(self):
        """
        True if site is an ordered site, i.e., with a single species with 
        occupancy 1.
        """
        return self._is_ordered

    @property
    def x(self):
        """
        Cartesian x coordinate
        """
        return self._coords[0]

    @property
    def y(self):
        """
        Cartesian y coordinate
        """
        return self._coords[1]

    @property
    def z(self):
        """
        Cartesian z coordinate
        """
        return self._coords[2]

    def __getitem__(self, el):
        '''
        Get the occupancy for element
        '''
        return self._species[el]

    def __eq__(self, other):
        """
        Site is equal to another site if the species and occupancies are the 
        same, and the coordinates are the same to some tolerance.  numpy
        function `allclose` is used to determine if coordinates are close.
        """
        if other == None:
            return False
        return self._species == other._species and \
               np.allclose(self._coords, other._coords)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        '''
        Minimally effective hash function that just distinguishes between Sites 
        with different elements.
        '''
        hashcode = 0
        for el in self._species.keys():
            #Ignore elements with zero amounts.
            if self[el] != 0:
                hashcode += el.Z
        return 7

    def __contains__(self, el):
        return el in self._species

    def __len__(self):
        return len(self._species)

    def __iter__(self):
        return self._species.__iter__()

    def __repr__(self):
        outs = []
        outs.append("Non-periodic Site")
        outs.append("xyz        : (%0.4f, %0.4f, %0.4f)" % tuple(self.coords))
        for k, v in self._species.items():
            outs.append("element    : %s" % k.symbol)
            outs.append("occupation : %0.2f" % v)
        return "\n".join(outs)

    def __cmp__(self, other):
        '''
        Sets a default sort order for atomic species by electronegativity. Very
        useful for getting correct formulas.  For example, FeO4PLi is 
        automatically sorted in LiFePO4.
        '''
        def avg_electroneg(sps):
            return sum([sp.X * occu for sp, occu in sps.items()])

        if avg_electroneg(self._species) < avg_electroneg(other._species):
            return -1
        if avg_electroneg(self._species) > avg_electroneg(other._species):
            return 1
        return 0

    def __str__(self):
        return "%s %s" % (self._coords, ','.join(["%s %.4f" % (str(atom), occu)\
                                     for atom, occu in self._species.items()]))

    @property
    def to_dict(self):
        """
        Json-serializable dict representation for Site.
        """
        species_list = []
        for spec, occu in self._species.items():
            d = spec.to_dict
            d['occu'] = occu
            species_list.append(d)
        d = {'name': self.species_string, 'species': species_list,
             'occu': occu, 'xyz':[float(c) for c in self._coords],
             'properties': self._properties}
        d['module'] = self.__class__.__module__
        d['class'] = self.__class__.__name__
        return d

    @staticmethod
    def from_dict(d):
        """
        Create Site from dict representation
        """
        atoms_n_occu = {}
        for sp_occu in d['species']:
            sp = Specie.from_dict(sp_occu) if 'oxidation_state' in sp_occu \
                 else Element(sp_occu['element'])
            atoms_n_occu[sp] = sp_occu['occu']
        props = d.get('properties', None)
        return Site(atoms_n_occu, d['xyz'], properties=props)


class PeriodicSite(Site, MSONable):
    """
    Extension of generic Site object to periodic systems.
    PeriodicSite includes a lattice system.
    """

    def __init__(self, atoms_n_occu, coords, lattice, to_unit_cell=False,
                 coords_are_cartesian=False, properties=None):
        """
        Create a periodic site.
        
        Args:
            atoms_n_occu:
                Species on the site. Can be:
                
                i.  A sequence of element / specie specified either as string
                    symbols, e.g. ["Li", "Fe2+", "P", ...] or atomic numbers,
                    e.g., (3, 56, ...) or actual Element or Specie objects.
                
                ii. List of dict of elements/species and occupancies, e.g.,
                    [{'Fe' : 0.5, 'Mn':0.5}, ...]. This allows the setup of
                    disordered structures.
            coords:
                coordinates of site as fractional coordinates or cartesian 
                coordinates.
            lattice:
                Lattice associated with the site
            to_unit_cell:
                translates fractional coordinate to the basic unit cell, i.e. 
                all fractional coordinates satisfy 0 <= a < 1. Defaults to 
                False.
            coords_are_cartesian:
                Set to True if you are providing cartesian coordinates. 
                Defaults to False.
            properties:
                Properties associated with the PeriodicSite as a dict, e.g. 
                {'magmom':5}. Defaults to None.
        """
        self._lattice = lattice
        self._fcoords = self._lattice.get_fractional_coords(coords) \
                        if coords_are_cartesian else coords

        if to_unit_cell:
            for i, fcoords in enumerate(self._fcoords):
                self._fcoords[i] = fcoords - math.floor(fcoords)

        c_coords = self._lattice.get_cartesian_coords(self._fcoords)
        Site.__init__(self, atoms_n_occu, c_coords, properties)

    @property
    def lattice(self):
        """
        The lattice associated with the site.
        """
        return self._lattice

    @property
    def frac_coords(self):
        """
        A copy of the fractional coordinates of the site.
        """
        return np.copy(self._fcoords)

    @property
    def a(self):
        """
        Fractional a coordinate
        """
        return self._fcoords[0]

    @property
    def b(self):
        """
        Fractional b coordinate
        """
        return self._fcoords[1]

    @property
    def c(self):
        """
        Fractional c coordinate
        """
        return self._fcoords[2]

    @property
    def to_unit_cell(self):
        """
        Copy of PeriodicSite translated to the unit cell.
        """
        return PeriodicSite(self._species, coords_to_unit_cell(self._fcoords),
                            self._lattice,
                            properties=self._properties)

    def is_periodic_image(self, other, tolerance=1e-8, check_lattice=True):
        """
        Returns True if sites are periodic images of each other.
        """
        if check_lattice and self.lattice != other.lattice:
            return False
        if self.species_and_occu != other.species_and_occu:
            return False
        frac_diff = abs(np.array(self._fcoords) - np.array(other._fcoords)) % 1
        frac_diff = [abs(a) < tolerance or abs(a) > 1 - tolerance \
                     for a in frac_diff]
        return  all(frac_diff)

    def __eq__(self, other):
        return self._species == other._species and \
            self._lattice == other._lattice and \
            np.allclose(self._coords, other._coords)

    def __ne__(self, other):
        return not self.__eq__(other)

    def distance_and_image_old(self, other, jimage=None):
        """
        .. deprecated:: 1.0
            Use :func:`distance_and_image` instead. This code is kept for 
            information reasons. A new version has been written which is more 
            accurate, but at a higher computational cost.
        
        Gets distance between two sites assuming periodic boundary conditions.
        If the index jimage of two sites atom j is not specified it selects the 
        j image nearest to the i atom and returns the distance and jimage
        indices in terms of lattice vector translations. If the index jimage of
        atom j is specified it returns the distance between the i atom and the
        specified jimage atom, the given jimage is also returned.
        
        Args:
            other:
                other site to get distance from.
            jimage:
                specific periodic image in terms of lattice translations, 
                e.g., [1,0,0] implies to take periodic image that is one 
                a-lattice vector away. If jimage == None, the image that is 
                nearest to the site is found.
        
        Returns:
            (distance, jimage):
                distance and periodic lattice translations of the other site 
                for which the distance applies.
        
        .. note::
            Assumes the primitive cell vectors are sufficiently not skewed such 
            that the condition \|a\|cos(ab_angle) < \|b\| for all possible cell
            vector pairs. ** this method does not check this condition **
        """
        if jimage == None:
            #Old algorithm
            jimage = -np.array(np.around(other._fcoords - self._fcoords), int)
        return np.linalg.norm(self.lattice.get_cartesian_coords(jimage + \
                                    other._fcoords - self._fcoords)), jimage

    def distance_and_image_from_frac_coords(self, fcoords, jimage=None):
        """
        Gets distance between site and a fractional coordinate assuming 
        periodic boundary conditions. If the index jimage of two sites atom j 
        is not specified it selects the j image nearest to the i atom and
        returns the distance and jimage indices in terms of lattice vector 
        translations. If the index jimage of atom j is specified it returns the
        distance between the i atom and the specified jimage atom, the given 
        jimage is also returned.
        
        Args:
            fcoords:
                fcoords to get distance from.
            jimage:
                specific periodic image in terms of lattice translations, 
                e.g., [1,0,0] implies to take periodic image that is one 
                a-lattice vector away. If jimage == None, the image that is
                nearest to the site is found.
        
        Returns:
            (distance, jimage):
                distance and periodic lattice translations of the other site
                for which the distance applies.
        """
        if jimage == None:
            #The following code is heavily vectorized to maximize speed.

            #Get the image adjustment necessary to bring coords to unit_cell.
            adj1 = np.array([-math.floor(i) for i in self._fcoords])
            adj2 = np.array([-math.floor(i) for i in fcoords])

            #Shift coords to unitcell
            coord1 = self._fcoords + adj1
            coord2 = fcoords + adj2

            #Generate set of images required for testing.
            test_set = [[-1, 0] if coord1[i] < coord2[i] else [0, 1] \
                        for i in range(3)]
            images = [image for image in itertools.product(*test_set)]

            #Create vectorized cartesian coords tiling for computing distances.
            vec = np.tile(coord2, (8, 1)) - np.tile(coord1, (8, 1)) + images
            vec = self._lattice.get_cartesian_coords(vec)

            #Compute distances manually.
            dist = np.sqrt(np.sum(vec ** 2, 1)).tolist()

            #Return the minimum distance and the adjusted image corresponding
            #to the min distance.
            mindist = min(dist)
            ind = dist.index(mindist)
            return mindist, adj2 - adj1 + images[ind]

        return np.linalg.norm(self.lattice.get_cartesian_coords(jimage + \
                                            fcoords - self._fcoords)), jimage

    def distance_and_image(self, other, jimage=None):
        """
        Gets distance and instance between two sites assuming periodic boundary 
        conditions. If the index jimage of two sites atom j is not specified it
        selects the j image nearest to the i atom and returns the distance and 
        jimage indices in terms of lattice vector translations. If the index
        jimage of atom j is specified it returns the distance between the ith
        atom and the specified jimage atom, the given jimage is also returned.
        
        Args:
            other:
                other site to get distance from.
            jimage:
                specific periodic image in terms of lattice translations, 
                e.g., [1,0,0] implies to take periodic image that is one 
                a-lattice vector away. If jimage == None, the image that is 
                nearest to the site is found.
        
        Returns:
            (distance, jimage):
                distance and periodic lattice translations of the other site 
                for which the distance applies.
        """
        return self.distance_and_image_from_frac_coords(other._fcoords, jimage)

    def distance(self, other, jimage=None):
        """
        Get distance between two sites assuming periodic boundary conditions.
        
        Args:
            other:
                other site to get distance from.
            jimage:   
                specific periodic image in terms of lattice translations, 
                e.g., [1,0,0] implies to take periodic image that is one
                a-lattice vector away. If jimage == None, the image that is
                nearest to the site is found.
        
        Returns:
            distance:
                distance between the two sites
        
        """
        return self.distance_and_image(other, jimage)[0]

    def __repr__(self):
        outs = []
        outs.append("Periodic Site")
        outs.append("abc : (%0.4f, %0.4f, %0.4f)" % tuple(self._fcoords))
        for k, v in self._species.items():
            outs.append("element    : %s" % k.symbol)
            outs.append("occupation : %0.2f" % v)
        return "\n".join(outs)

    @property
    def to_dict(self):
        """
        Json-serializable dict representation of PeriodicSite.
        """
        species_list = []
        for spec, occu in self._species.items():
            d = spec.to_dict
            d['occu'] = occu
            species_list.append(d)
        d = {'label': self.species_string, 'species': species_list,
             'occu': occu, 'xyz':[float(c) for c in self._coords],
             'abc':[float(c) for c in self._fcoords],
             'lattice': self._lattice.to_dict,
             'properties': self._properties}
        d['module'] = self.__class__.__module__
        d['class'] = self.__class__.__name__
        return d

    @staticmethod
    def from_dict(d, lattice=None):
        """
        Create PeriodicSite from dict representation.
        
        Args:
            d:
                dict representation of PeriodicSite
            lattice:
                Optional lattice to override lattice specified in d. Useful for
                ensuring all sites in a structure share the same lattice.
        """
        atoms_n_occu = {}
        for sp_occu in d['species']:
            sp = Specie.from_dict(sp_occu) if 'oxidation_state' in sp_occu \
                 else Element(sp_occu['element'])
            atoms_n_occu[sp] = sp_occu['occu']
        props = d.get('properties', None)
        lattice = lattice if lattice else Lattice.from_dict(d['lattice'])
        return PeriodicSite(atoms_n_occu, d['abc'], lattice, properties=props)



import os
from distutils.core import setup, Extension
from numpy.distutils.misc_util import get_numpy_include_dirs

spgsrcdir = os.path.join('extensions', 'spglib-1.2', 'src')

include_dirs = [spgsrcdir]
sources = ['cell.c', 'debug.c', 'hall_symbol.c', 'kpoint.c', 'lattice.c',
    'mathfunc.c', 'pointgroup.c', 'primitive.c', 'refinement.c',
    'sitesym_database.c', 'site_symmetry.c', 'spacegroup.c', 'spin.c',
    'spg_database.c', 'spglib.c', 'symmetry.c'
]

sources = [os.path.join(spgsrcdir, srcfile) for srcfile in sources]

extension = Extension('pyspglib._spglib',
                      include_dirs=include_dirs + get_numpy_include_dirs(),
                      sources=['_spglib.c'] + sources,
                      extra_compile_args=['-fopenmp'],
                      extra_link_args=['-lgomp'],
                      )

setup (
       name='spglib',
       version='1.2',
       description='This is the spglib module.',
       author='Atsushi Togo',
       author_email='atz.togo@gmail.com',
       url='http://spglib.sourceforge.net/',
       packages=['pyspglib'],
       ext_modules=[extension]
)



#!/usr/bin/env python

"""
This module provides classes to create phase diagrams.
"""

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "2.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "Jul 11, 2012"

import numpy as np
import collections
import logging

from pymatgen.core.structure import Composition
from pymatgen.phasediagram.entries import GrandPotPDEntry, TransformedPDEntry
from pymatgen.util.coord_utils import get_convex_hull
from pymatgen.core.periodic_table import DummySpecie
from pymatgen.analysis.reaction_calculator import Reaction, ReactionError


logger = logging.getLogger(__name__)


class PhaseDiagram (object):
    '''
    Simple phase diagram class taking in elements and entries as inputs.
    The algorithm is based on the work in the following papers:
    
    1. S. P. Ong, L. Wang, B. Kang, and G. Ceder, Li-Fe-P-O2 Phase Diagram from
       First Principles Calculations. Chem. Mater., 2008, 20(5), 1798-1807.
       doi:10.1021/cm702327g
    
    2. S. P. Ong, A. Jain, G. Hautier, B. Kang, G. Ceder, Thermal stabilities
       of delithiated olivine MPO4 (M=Fe, Mn) cathodes investigated using first
       principles calculations. Electrochem. Comm., 2010, 12(3), 427-430. 
       doi:10.1016/j.elecom.2010.01.010
    '''
    FORMATION_ENERGY_TOLERANCE = 1e-11

    def __init__(self, entries, elements=None, use_external_qhull=False):
        """
        Standard constructor for phase diagram.
        
        Args:
            entries:
                A list of PDEntry-like objects having an energy, energy_per_atom
                and composition.
            elements:
                Optional list of elements in the phase diagram. If set to None,
                the elements are determined from the the entries themselves.
            use_external_qhull:
                If set to True, the code will use an external command line call
                to Qhull to calculate the convex hull data. This requires the
                user to have qhull installed and the executables 'qconvex'
                available in his path. By default, the code uses the
                scipy.spatail.Delaunay.
                
                The benefit of using external qhull is that a) it is much
                faster, especially for higher-dimensional hulls with many
                entries, and b) it is more robustly tested. The scipy Delaunay
                class is relatively new, and we have found some issues with
                higher-dimensional hulls, with non-sensical results given in
                some instances. Nonetheless, scipy Delaunay seems to work well
                enough for phase diagrams with < 4 components.
        """
        if elements == None:
            elements = set()
            map(elements.update, [entry.composition.elements for entry in entries])
        self._all_entries = entries
        self._elements = tuple(elements)
        self._qhull_data = None
        self._facets = None
        self._qhull_entries = None
        self._stable_entries = None
        self._use_external_qhull = use_external_qhull
        self._make_phasediagram()

    @property
    def all_entries(self):
        """
        All entries provided for Phase Diagram construction. Note that this
        does not mean that all these entries are actually used in the phase
        diagram. For example, this includes the positive formation energy
        entries that are filtered out before Phase Diagram construction.
        """
        return self._all_entries

    @property
    def dim(self):
        """
        The dimensionality of the phase diagram
        """
        return len(self._elements)

    @property
    def elements(self):
        """
        Elements in the phase diagram.
        """
        return self._elements

    @property
    def facets(self):
        """
        Facets of the phase diagram in the form of  [[1,2,3],[4,5,6]...]
        """
        return self._facets

    @property
    def qhull_data(self):
        """
        Data used in the convex hull operation. This is essentially a matrix of
        composition data and energy per atom values created from qhull_entries.
        """
        return self._qhull_data

    @property
    def qhull_entries(self):
        """
        Actual entries used in convex hull. Excludes all positive formation
        energy entries.
        """
        return self._qhull_entries

    @property
    def unstable_entries(self):
        """
        Entries that are unstable in the phase diagram. Includes positive
        formation energy entries.
        """
        return [e for e in self.all_entries if e not in self.stable_entries]

    @property
    def stable_entries(self):
        '''
        Returns the stable entries in the phase diagram.
        '''
        return self._stable_entries

    @property
    def all_entries_hulldata(self):
        """
        Same as qhull_data, but for all entries rather than just positive
        formation energy ones.
        """
        return self._process_entries_qhulldata(self._all_entries)

    @property
    def el_refs(self):
        """
        List of elemental references for the phase diagrams. These are
        entries corresponding to the lowest energy element entries for simple
        compositional phase diagrams.
        """
        return self._el_refs

    def get_form_energy(self, entry):
        '''
        Returns the formation energy for an entry (NOT normalized) from the
        elemental references.
        
        Args:
            entry:
                A PDEntry-like object.
        
        Returns:
            Formation energy from the elemental references.
        '''
        comp = entry.composition
        energy = entry.energy - sum([comp[el] * self._el_refs[el].energy_per_atom for el in comp.elements])
        return energy

    def get_form_energy_per_atom(self, entry):
        '''
        Returns the formation energy per atom for an entry from the
        elemental references.
        
        Args:
            entry:
                An PDEntry-like object
        
        Returns:
            Formation energy **per atom** from the elemental references.
        '''
        comp = entry.composition
        return self.get_form_energy(entry) / comp.num_atoms

    def _process_entries_qhulldata(self, entries_to_process):
        """
        From a sequence of entries, generate the necessary for the convex hull.
        Using the Li-Fe-O phase diagram as an example, this is of the form:
        [[ Fe_fraction_entry_1, O_fraction_entry_1, Energy_per_atom_entry_1],
         [ Fe_fraction_entry_2, O_fraction_entry_2, Energy_per_atom_entry_2],
         ...]]
        
        Note that there are only two independent variables, since the third
        elemental fraction is fixed by the constraint that all compositions sum
        to 1. The choice of the elements is arbitrary.
        """
        data = []
        for entry in entries_to_process:
            comp = entry.composition
            energy_per_atom = entry.energy_per_atom
            row = []
            for i in xrange(1, len(self._elements)):
                row.append(comp.get_atomic_fraction(self._elements[i]))
            row.append(energy_per_atom)
            data.append(row)
        return data

    def _create_convhull_data(self):
        '''
        Make data suitable for convex hull procedure from the list of entries.
        The procedure is as follows:
        
        1. First find the elemental references, i.e., the lowest energy entry
           for the vertices of the phase diagram. Using the Li-Fe-O phase
           diagram as an example, this means the lowest energy Li, Fe, and O
           phases.
        
        2. Calculate the formation energies from these elemental references for
           all entries. Exclude all positive formation energy ones from the data
           for convex hull.
        
        3. Generate the convex hull data.
        '''
        logger.debug("Creating convex hull data...")
        #Determine the elemental references based on lowest energy for each.
        self._el_refs = dict()
        for entry in self._all_entries:
            if entry.composition.is_element:
                for el in entry.composition.elements:
                    if entry.composition[el] > Composition.amount_tolerance:
                        break
                e_per_atom = entry.energy_per_atom
                if el not in self._el_refs:
                    self._el_refs[el] = entry
                elif self._el_refs[el].energy_per_atom > e_per_atom:
                    self._el_refs[el] = entry
        # Remove positive formation energy entries
        entries_to_process = list()
        for entry in self._all_entries:
            if self.get_form_energy(entry) <= -self.FORMATION_ENERGY_TOLERANCE:
                entries_to_process.append(entry)
            else:
                logger.debug("Removing positive formation energy entry {}".format(entry))
        entries_to_process.extend([entry for entry in self._el_refs.values()])

        self._qhull_entries = entries_to_process
        return self._process_entries_qhulldata(entries_to_process)

    def _make_phasediagram(self):
        """
        Make the phase diagram.
        """
        stable_entries = set()
        dim = len(self._elements)
        self._qhull_data = self._create_convhull_data()
        if len(self._qhull_data) == dim:
            self._facets = [range(len(self._elements))]
        else:
            self._facets = get_convex_hull(self._qhull_data,
                                           self._use_external_qhull)
            logger.debug("Final facets are\n{}".format(self._facets))

            logger.debug("Removing vertical facets...")
            finalfacets = list()
            for facet in self._facets:
                facetmatrix = np.zeros((len(facet), len(facet)))
                count = 0
                is_element_facet = True
                for vertex in facet:
                    facetmatrix[count] = np.array(self._qhull_data[vertex])
                    facetmatrix[count, dim - 1] = 1
                    count += 1
                    if len(self._qhull_entries[vertex].composition) > 1:
                        is_element_facet = False
                if abs(np.linalg.det(facetmatrix)) > 1e-8 and \
                   (not is_element_facet):
                    finalfacets.append(facet)
                else:
                    logger.debug("Removing vertical facet : {}".format(facet))
            self._facets = finalfacets

        for facet in self._facets:
            for vertex in facet:
                stable_entries.add(self._qhull_entries[vertex])
        self._stable_entries = stable_entries

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        symbols = [el.symbol for el in self._elements]
        output = []
        output.append("{} phase diagram".format("-".join(symbols)))
        output.append("{} stable phases: ".format(len(self._stable_entries)))
        output.append(", ".join([entry.name for entry in self._stable_entries]))
        return "\n".join(output)


class GrandPotentialPhaseDiagram(PhaseDiagram):
    '''
    A class representing a Grand potential phase diagram. Grand potential phase
    diagrams are essentially phase diagrams that are open to one or more
    components. To construct such phase diagrams, the relevant free energy is
    the grand potential, which can be written as the Legendre transform of the
    Gibbs free energy as follows
     
    Grand potential = G - u\ :sub:`X` N\ :sub:`X`\ 
    
    The algorithm is based on the work in the following papers:
    
    1. S. P. Ong, L. Wang, B. Kang, and G. Ceder, Li-Fe-P-O2 Phase Diagram from
       First Principles Calculations. Chem. Mater., 2008, 20(5), 1798-1807.
       doi:10.1021/cm702327g
    
    2. S. P. Ong, A. Jain, G. Hautier, B. Kang, G. Ceder, Thermal stabilities
       of delithiated olivine MPO4 (M=Fe, Mn) cathodes investigated using first
       principles calculations. Electrochem. Comm., 2010, 12(3), 427-430. 
       doi:10.1016/j.elecom.2010.01.010
    '''

    def __init__(self, entries, chempots, elements=None,
                 use_external_qhull=False):
        """
        Standard constructor for grand potential phase diagram.
        
        Args:
            entries:
                A list of PDEntry-like objects having an energy, energy_per_atom
                and composition.
            chempots:
                A dict of {element: float} to specify the chemical potentials
                of the open elements.
            elements:
                Optional list of elements in the phase diagram. If set to None,
                the elements are determined from the entries themselves.
            use_external_qhull:
                If set to True, the code will use an external command line call
                to Qhull to calculate the convex hull data instead of
                scipy.spatial.Delaunay. See the doc for the PhaseDiagram class
                for an explanation of the pros and cons.
        """
        if elements == None:
            elements = set()
            map(elements.update, [entry.composition.elements for entry in entries])
        allentries = list()
        for entry in entries:
            if not (entry.is_element and (entry.composition.elements[0] in chempots)):
                allentries.append(GrandPotPDEntry(entry, chempots))
        self.chempots = chempots
        filteredels = list()
        for el in elements:
            if el not in chempots:
                filteredels.append(el)
        elements = sorted(filteredels)
        super(GrandPotentialPhaseDiagram, self).__init__(allentries, elements,
                                                         use_external_qhull)

    def __str__(self):
        symbols = [el.symbol for el in self._elements]
        output = []
        output.append("{} grand potential phase diagram with ".format("-".join(symbols)))
        output[-1] += ", ".join(["u{}={}".format(el, v) for el, v in self.chempots.items()])
        output.append("{} stable phases: ".format(len(self._stable_entries)))
        output.append(", ".join([entry.name for entry in self._stable_entries]))
        return "\n".join(output)



class CompoundPhaseDiagram(PhaseDiagram):
    """
    Generates phase diagrams from compounds as terminations instead of elements.
    """

    def __init__(self, entries, terminal_compositions,
                 use_external_qhull=False):
        """
        Args:
            entries:
                Sequence of input entries. For example, if you want a Li2O-P2O5
                phase diagram, you might have all Li-P-O entries as an input.
            terminal_compositions:
                Terminal compositions of phase space. In the Li2O-P2O5 example,
                these will be the Li2O and P2O5 compositions.
            use_external_qhull:
                Similar to PhaseDiagram, set to True if you wish to use external
                qhull command.
        """
        self.original_entries = entries
        self.terminal_compositions = terminal_compositions
        (pentries, species_mapping) = self.transform_entries(entries, terminal_compositions)
        PhaseDiagram.__init__(self, pentries, elements=species_mapping.values(),
                              use_external_qhull=use_external_qhull)

    def transform_entries(self, entries, terminal_compositions):
        """
        Method to transform all entries to the composition coordinate in the
        terminal compositions. If the entry does not fall within the space
        defined by the terminal compositions, they are excluded. For example,
        Li3PO4 is mapped into a Li2O:1.5, P2O5:0.5 composition. The terminal
        compositions are represented by DummySpecies.
        
        Args:
            entries:
                Sequence of all input entries
            terminal_compositions:
                Terminal compositions of phase space.
        
        Returns:
            Sequence of TransformedPDEntries falling within the phase space.
        """
        new_entries = []
        #Map terminal compositions to unique dummy species.
        sp_mapping = collections.OrderedDict()
        for i, comp in enumerate(terminal_compositions):
            sp_mapping[comp] = DummySpecie('X' + chr(102 + i))

        for entry in entries:
            try:
                rxn = Reaction(terminal_compositions, [entry.composition])
                rxn.normalize_to(entry.composition)
                #We only allow reactions that have positive amounts of reactants.
                if all([rxn.get_coeff(comp) <= 1e-5 for comp in terminal_compositions]):
                    newcomp = {sp_mapping[comp]:-rxn.get_coeff(comp) \
                               for comp in terminal_compositions}
                    newcomp = {k: v for k, v in newcomp.items() if v > 1e-5}
                    transformed_entry = TransformedPDEntry(Composition(newcomp),
                                                           entry)
                    new_entries.append(transformed_entry)
            except ReactionError as ex:
                #If the reaction can't be balanced, the entry does not fall
                #into the phase space. We ignore them.
                pass
        return (new_entries, sp_mapping)





#!/usr/bin/env python

"""
This module provides classes to perform topological analyses of structures.
"""

from __future__ import division

__author__ = "Shyue Ping Ong, Geoffroy Hautier"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "$Sep 23, 2011M$"

import math
import numpy as np
import itertools

from pymatgen.command_line.qhull_caller import qvoronoi, qvertex


class VoronoiCoordFinder:
    """
    Uses a Voronoi algorithm to determine the coordination for each site in a
    structure.
    """

    """Radius in Angstrom cutoff to look for coordinating atoms"""
    default_cutoff = 10.0

    def __init__(self, structure, target=None):
        """
        Args:
            structure:
                Input structure
            target:
                A list of target species to determine coordination for.
        """
        self._structure = structure
        if target == None:
            self._target = structure.composition.elements
        else:
            self._target = target

    def get_voronoi_polyhedra(self, n):
        """
        Gives a weighted polyhedra around a site. This uses the voronoi 
        construction with solid angle weights.
        See ref: A Proposed Rigorous Definition of Coordination Number,
        M. O'Keeffe, Acta Cryst. (1979). A35, 772-775
        
        Args: 
            n:
                site index
        
        Returns:
            A dictionary of sites sharing a common Voronoi facet with the site
            n and their solid angle weights
        """

        localtarget = self._target
        center = self._structure[n]
        neighbors = self._structure.get_sites_in_sphere(center.coords,
                                            VoronoiCoordFinder.default_cutoff)
        neighbors = [i[0] for i in sorted(neighbors, key=lambda s : s[1])]
        qvoronoi_input = [s.coords for s in neighbors]
        closest = qvoronoi(qvoronoi_input)
        all_vertices = qvertex(qvoronoi_input)

        result = []
        angle = []
        for i in range(len(all_vertices)):
            if closest[i][1] == 0:
                facets = []
                result.append(neighbors[closest[i][2]])
                for j in range(3, len(closest[i])):
                    if closest[i][j] == 0:
                        raise RuntimeError("This structure is pathological, infinite vertex in the voronoi construction ");
                    facets.append(all_vertices[closest[i][j] - 1])

                angle.append(solid_angle(center.coords, facets))

        maxangle = max(angle)

        resultweighted = {}
        for j in range(len(angle)):
            if result[j].specie in localtarget:
                resultweighted[result[j]] = angle[j] / maxangle

        return resultweighted

    def get_coordination_number(self, n):
        """
        Returns the coordination number of site with index n.
        
        Args: 
            n:
                site index
        """
        return sum(self.get_voronoi_polyhedra(n).values())

    def get_coordinated_sites(self, n, tol=0, target=None):
        """
        Returns the sites that are in the coordination radius of site with index n.
        
        Args: 
            n:
                Site number.
            tol:
                Weight tolerance to determine if a particular pair is
                considered a neighbor.
            Target:
                Target element
            
        Returns:
            Sites coordinating input site.
        """
        coordinated_sites = []
        for site, weight in self.get_voronoi_polyhedra(n).items():
            if weight > tol and (target == None or site.specie == target):
                coordinated_sites.append(site)
        return coordinated_sites


def solid_angle(center, coords):
    """
    Helper method to calculate the solid angle of a set of coords from the
    center.
    
    Args:
        center:
            Center to measure solid angle from.
        coords:
            List of coords to determine solid angle.
            
    Returns:
        The solid angle.
    """
    o = np.array(center)
    r = [np.array(c) - o for c in coords]
    r.append(r[0])
    n = [np.cross(r[i + 1], r[i]) for i in range(len(r) - 1)]
    n.append(np.cross(r[1], r[0]))
    phi = sum([math.acos(-np.dot(n[i], n[i + 1]) / (np.linalg.norm(n[i]) * np.linalg.norm(n[i + 1]))) for i in range(len(n) - 1)])
    return phi + (3 - len(r)) * math.pi



def contains_peroxide(structure, relative_cutoff=1.2):
    """
    Determines if a structure contains peroxide anions.
    
    Args:
        structure:
            Input structure.
        relative_cutoff:
            The peroxide bond distance is 1.49 Angstrom. Relative_cutoff * 1.49
            stipulates the maximum distance two O atoms must be to each other
            to be considered a peroxide. 
    
    Returns:
        Boolean indicating if structure contains a peroxide anion.
    """
    max_dist = relative_cutoff * 1.49
    o_sites = []
    for site in structure:
        syms = [sp.symbol for sp in site.species_and_occu.keys()]
        if "O" in syms:
            o_sites.append(site)

    for i, j in itertools.combinations(o_sites, 2):
        if i.distance(j) < max_dist:
            return True

    return False




#!/usr/bin/env python

"""
This module provides classes for plotting PhaseDiagram objects.
"""

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ = "Jun 15, 2012"

import math
import numpy as np
import itertools

from pymatgen.phasediagram.pdanalyzer import PDAnalyzer
from pymatgen.util.string_utils import latexify
from pymatgen.util.coord_utils import get_convex_hull


class PDPlotter(object):
    '''
    A plotter class for phase diagrams.
    '''

    def __init__(self, phasediagram, show_unstable=False):
        """
        Args:
            phasediagram:
                A PhaseDiagram object.
            show_unstable:
                Whether unstable phases will be plotted as well as red crosses.
                Defaults to False.
        """
        self._pd = phasediagram
        self._dim = len(self._pd.elements)
        self.lines = uniquelines(self._pd.facets)
        self.show_unstable = show_unstable
        if self._dim < 2 or self._dim > 4:
            raise ValueError("Only 2-4 components supported!")

    @property
    def pd_plot_data(self):
        '''
        Plot data for phase diagram.
        2-comp - Full hull with energies
        3/4-comp - Projection into 2D or 3D gibbs triangle.
        
        Returns:
            (lines, stable_entries, unstable_entries):
                - lines is a list of list of coordinates for lines in the PD. 
                - stable_entries is a {coordinate : entry} for each stable node 
                  in the phase diagram. (Each coordinate can only have one 
                  stable phase)
                - unstable_entries is a {entry: coordinates} for all unstable
                  nodes in the phase diagram.
        '''
        pd = self._pd
        entries = pd.qhull_entries
        data = np.array(pd.qhull_data)
        facetlines = self.lines
        lines = list()
        stable_entries = dict()
        for line in facetlines:
            entry1 = entries[line[0]]
            entry2 = entries[line[1]]
            if self._dim == 2:
                x = [data[line[0]][0], data[line[1]][0]]
                y = [pd.get_form_energy_per_atom(entry1),
                     pd.get_form_energy_per_atom(entry2)]
                coord = [x, y]
            elif self._dim == 3:
                coord = triangular_coord(data[line, 0:2])
            else:
                coord = tet_coord(data[line, 0:3])
            lines.append(coord)
            labelcoord = list(zip(*coord))
            stable_entries[labelcoord[0]] = entry1
            stable_entries[labelcoord[1]] = entry2

        allentries = pd.all_entries
        alldata = np.array(pd.all_entries_hulldata)
        unstable_entries = dict()
        stable = pd.stable_entries
        for i in xrange(0, len(allentries)):
            entry = allentries[i]
            if entry not in stable:
                if self._dim == 2:
                    x = [alldata[i][0], alldata[i][0]]
                    y = [pd.get_form_energy_per_atom(entry),
                         pd.get_form_energy_per_atom(entry)]
                    coord = [x, y]
                elif self._dim == 3:
                    coord = triangular_coord([alldata[i, 0:2], alldata[i, 0:2]])
                else:
                    coord = tet_coord([alldata[i, 0:3], alldata[i, 0:3],
                                       alldata[i, 0:3]])
                labelcoord = list(zip(*coord))
                unstable_entries[entry] = labelcoord[0]

        return (lines, stable_entries, unstable_entries)

    def show(self):
        """
        Draws the phase diagram using Matplotlib and show it.
        """
        if self._dim < 4:
            plt = self._get_2d_plot()
        elif self._dim == 4:
            plt = self._get_3d_plot()
        plt.show()

    def _get_2d_plot(self):
        '''
        Shows the plot using pylab.  Usually I won't do imports in methods,
        but since plotting is a fairly expensive library to load and not all 
        machines have matplotlib installed, I have done it this way.
        '''
        from pymatgen.util.plotting_utils import get_publication_quality_plot
        plt = get_publication_quality_plot(8, 6)
        from matplotlib.font_manager import FontProperties
        (lines, labels, unstable) = self.pd_plot_data
        for x, y in lines:
            plt.plot(x, y, 'ko-', linewidth=3, markeredgecolor='k',
                     markerfacecolor='b', markersize=10)
        font = FontProperties()
        font.set_weight('bold')
        font.set_size(24)

        # Sets a nice layout depending on the type of PD. Also defines a
        # "center" for the PD, which then allows the annotations to be spread
        # out in a nice manner.
        if len(self._pd.elements) == 3:
            plt.axis('equal')
            plt.xlim((-0.1, 1.2))
            plt.ylim((-0.1, 1.0))
            plt.axis('off')
            center = (0.5, math.sqrt(3) / 6)
        else:
            allcoords = labels.keys()
            miny = min([c[1] for c in allcoords])
            ybuffer = max(abs(miny) * 0.1, 0.1)
            plt.xlim((-0.1, 1.1))
            plt.ylim((miny - ybuffer, ybuffer))
            center = (0.5, miny / 2)
            plt.xlabel("Fraction")
            plt.ylabel("Formation energy from end members (eV)")

        for coords in sorted(labels.keys(), key=lambda x:-x[1]):
            entry = labels[coords]
            label = entry.name

            # The follow defines an offset for the annotation text emanating
            # from the center of the PD. Results in fairly nice layouts for the
            # most part.
            vec = (np.array(coords) - center)
            vec = vec / np.linalg.norm(vec) * 10
            valign = "bottom" if vec[1] > 0 else "top"
            if vec[0] < -0.01:
                halign = "right"
            elif vec[0] > 0.01:
                halign = "left"
            else:
                halign = "center"

            plt.annotate(latexify(label), coords, xytext=vec,
                             textcoords='offset points',
                             horizontalalignment=halign,
                             verticalalignment=valign,
                             fontproperties=font)

        if self.show_unstable:
            font = FontProperties()
            font.set_size(16)
            for entry, coords in unstable.items():
                vec = (np.array(coords) - center)
                vec = vec / np.linalg.norm(vec) * 10
                plt.plot(coords[0], coords[1], 'rx', linewidth=3,
                         markeredgecolor='r', markerfacecolor='r',
                         markersize=10)
                plt.annotate(latexify(entry.name), coords, xytext=vec,
                             textcoords='offset points',
                             horizontalalignment=halign, color="b",
                             verticalalignment=valign,
                             fontproperties=font)
        F = plt.gcf()
        F.set_size_inches((8, 6))
        plt.subplots_adjust(left=0.09, right=0.98, top=0.98, bottom=0.07)
        return plt

    def _get_3d_plot(self):
        '''
        Shows the plot using pylab.  Usually I won't do imports in methods,
        but since plotting is a fairly expensive library to load and not all 
        machines have matplotlib installed, I have done it this way.
        '''
        import matplotlib.pyplot as plt
        import mpl_toolkits.mplot3d.axes3d as p3
        from matplotlib.font_manager import FontProperties
        fig = plt.figure()
        ax = p3.Axes3D(fig)
        font = FontProperties()
        font.set_weight('bold')
        font.set_size(20)
        (lines, labels, unstable) = self.pd_plot_data
        count = 1
        newlabels = list()
        for x, y, z in lines:
            ax.plot(x, y, z, 'bo-', linewidth=3, markeredgecolor='b',
                    markerfacecolor='r', markersize=10)
        for coords in sorted(labels.keys()):
            entry = labels[coords]
            label = entry.name
            if len(entry.composition.elements) == 1:
                # commented out options are only for matplotlib 1.0.  Removed
                # them so that most ppl can use this class.
                ax.text(coords[0], coords[1], coords[2], label)#, horizontalalignment=halign, verticalalignment=valign, fontproperties=font)
            else:
                ax.text(coords[0], coords[1], coords[2], str(count))#, horizontalalignment=halign, verticalalignment=valign, fontproperties=font)
                newlabels.append("{} : {}".format(count, latexify(label)))
                count += 1
        plt.figtext(0.01, 0.01, '\n'.join(newlabels))
        ax.axis('off')
        return plt

    def write_image(self, stream, image_format="svg"):
        '''
        Writes the phase diagram to an image in a stream.
        
        Args:
            stream:
                stream to write to. Can be a file stream or a StringIO stream.
            image_format
                format for image. Can be any of matplotlib supported formats. 
                Defaults to svg for best results for vector graphics.
        '''
        if self._dim < 4:
            plt = self._get_2d_plot()
        elif self._dim == 4:
            plt = self._get_3d_plot()

        f = plt.gcf()
        f.set_size_inches((12, 10))

        plt.savefig(stream, format=image_format)


    def plot_chempot_range_map(self, elements):
        """
        Plot chemical potential range map. Currently works only for 3-component
        PDs.

        Args:
            elements:
                Sequence of elements to be considered as independent variables.
                E.g., if you want to show the stability ranges of all Li-Co-O
                phases wrt to uLi and uO, you will supply
                [Element("Li"), Element("O")]
        """
        from pymatgen.util.plotting_utils import get_publication_quality_plot
        from pymatgen.util.coord_utils import in_coord_list
        plt = get_publication_quality_plot(12, 8)
        analyzer = PDAnalyzer(self._pd)
        chempot_ranges = analyzer.get_chempot_range_map(elements)
        missing_lines = {}
        all_coords = []
        for entry, lines in chempot_ranges.items():
            center_x = 0
            center_y = 0
            coords = []
            for line in lines:
                (x, y) = line.coords.transpose()
                plt.plot(x, y, 'k')
                center_x += sum(x)
                center_y += sum(y)
                for coord in line.coords:
                    if not in_coord_list(coords, coord):
                        coords.append(coord.tolist())
                    else:
                        coords.remove(coord.tolist())
                all_coords.extend(line.coords)
            comp = entry.composition
            frac_sum = sum([comp.get_atomic_fraction(el) for el in elements])
            if coords and frac_sum < 0.99:
                missing_lines[entry] = coords
            else:
                xy = (center_x / 2 / len(lines), center_y / 2 / len(lines))
                plt.annotate(latexify(entry.name), xy, fontsize=22)

        ax = plt.gca()
        xlim = ax.get_xlim()
        ylim = ax.get_ylim()

        #This is a bit hacky, but it's a way to shade the forbidden chemical
        #potential regions.
        all_coords.append([xlim[0], ylim[1]])
        all_coords.append([xlim[1], ylim[0]])
        all_coords.append([xlim[0], ylim[0]])
        facets = get_convex_hull(all_coords)
        excluded_boundary = []
        for f in facets:
            if f[0] == len(all_coords) - 1 or f[1] == len(all_coords) - 1:
                continue
            if (all_coords[f[0]][0] != xlim[1] or all_coords[f[1]][0] != xlim[1]) and (all_coords[f[0]][1] != ylim[1] or all_coords[f[1]][1] != ylim[1]):
                excluded_boundary.append(tuple(all_coords[f[0]]))
                excluded_boundary.append(tuple(all_coords[f[1]]))
        excluded_boundary = list(set(excluded_boundary))
        excluded_boundary = sorted(excluded_boundary, key=lambda c: c[0])
        excluded_boundary.append((xlim[1], ylim[1]))
        x = [c[0] for c in excluded_boundary]
        y = [c[1] for c in excluded_boundary]
        plt.fill(x, y, '0.80')

        for entry, coords in missing_lines.items():
            center_x = 0
            center_y = 0
            comp = entry.composition
            if not comp.is_element:
                for coord in coords:
                    x = None
                    y = None
                    if entry.composition.get_atomic_fraction(elements[0]) < 0.01:
                        x = [coord[0], min(xlim)]
                        y = [coord[1], coord[1]]
                    elif entry.composition.get_atomic_fraction(elements[1]) < 0.01:
                        x = [coord[0], coord[0]]
                        y = [coord[1], min(ylim)]
                    if x and y:
                        plt.plot(x, y, 'k')
                        center_x += sum(x)
                        center_y += sum(y)
            else:
                center_x = sum(coord[0] for coord in coords) * 2 + xlim[0]
                center_y = sum(coord[1] for coord in coords) * 2 + ylim[0]
            xy = (center_x / 2 / len(coords), center_y / 2 / len(coords))
            plt.annotate(latexify(entry.name), xy, horizontalalignment="center",
                         verticalalignment="center", fontsize=22)

        plt.xlabel("$\mu_{{{0}}} - \mu_{{{0}}}^0$ (eV)".format(elements[0].symbol))
        plt.ylabel("$\mu_{{{0}}} - \mu_{{{0}}}^0$ (eV)".format(elements[1].symbol))
        plt.tight_layout()
        plt.show()


def uniquelines(q):
    '''
    Given all the facets, convert it into a set of unique lines.  Specifically
    used for converting convex hull facets into line pairs of coordinates.
    
    Args:
        q:
            A 2-dim sequence, where each row represents a facet. E.g., 
            [[1,2,3],[3,6,7],...]
    
    Returns:
        setoflines:
            A set of tuple of lines.  E.g., ((1,2), (1,3), (2,3), ....)
    '''
    setoflines = set()
    for facets in q:
        for line in itertools.combinations(facets, 2):
            setoflines.add(tuple(sorted(line)))
    return setoflines


def triangular_coord(coord):
    '''
    Convert a 2D coordinate into a triangle-based coordinate system for a
    prettier phase diagram.
    
    Args:
        coordinate:
            coordinate used in the convex hull computation.
    
    Returns:
        coordinates in a triangular-based coordinate system. 
    '''
    unitvec = np.array([[1, 0], [0.5, math.sqrt(3) / 2]])
    result = np.dot(np.array(coord), unitvec)
    return result.transpose()

def tet_coord(coord):
    '''
    Convert a 3D coordinate into a tetrahedron based coordinate system for a
    prettier phase diagram.
    
    Args:
        coordinate:
            coordinate used in the convex hull computation.
    
    Returns:
        coordinates in a tetrahedron-based coordinate system. 
    '''
    unitvec = np.array([[1, 0, 0], [0.5, math.sqrt(3) / 2, 0],
                        [0.5, 1.0 / 3.0 * math.sqrt(3) / 2, math.sqrt(6) / 3]])
    result = np.dot(np.array(coord), unitvec)
    return result.transpose()



#!/usr/bin/env python

'''
This module implements plotter for DOS.
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "May 1, 2012"


from collections import OrderedDict

import numpy as np
import logging
import math

from pymatgen.electronic_structure.core import Spin
from pymatgen.util.io_utils import clean_json

logger = logging.getLogger('BSPlotter')


class DosPlotter(object):
    """
    Class for plotting DOSes.
    """

    def __init__(self, zero_at_efermi=True, stack=False, sigma=None):
        """
        Args:
            zero_at_efermi:
                Whether to shift all Dos to have zero energy at the fermi energy.
                Defaults to True.
            stack:
                Whether to plot the DOS as a stacked area graph
            key_sort_func:
                function used to sort the dos_dict keys.
            sigma:
                A float specifying a standard deviation for Gaussian smearing the 
                DOS for nicer looking plots. Defaults to None for no smearing.
        """
        self.zero_at_efermi = zero_at_efermi
        self.stack = stack
        self.sigma = sigma
        self._doses = OrderedDict()

    def add_dos(self, label, dos):
        """
        Adds a dos for plotting.
        
        Args:
            label:
                label for the DOS. Must be unique.
            dos:
                Dos object
        """
        energies = dos.energies - dos.efermi if self.zero_at_efermi else dos.energies
        densities = dos.get_smeared_densities(self.sigma) if self.sigma else dos.densities
        efermi = dos.efermi
        self._doses[label] = {'energies':energies, 'densities': densities, 'efermi':efermi}

    def add_dos_dict(self, dos_dict, key_sort_func=None):
        """
        Add a dictionary of doses, with an optional sorting function for the 
        keys.
        
        Args:
            dos_dict:
                dict of {label: Dos}
            key_sort_func:
                function used to sort the dos_dict keys.
        """
        if key_sort_func:
            keys = sorted(dos_dict.keys(), key=key_sort_func)
        else:
            keys = dos_dict.keys()
        for label in keys:
            self.add_dos(label, dos_dict[label])

    def get_dos_dict(self):
        """
        Returns the added doses as a json-serializable dict. Note that if you
        have specified smearing for the DOS plot, the densities returned will
        be the smeared densities, not the original densities.
        
        Returns:
            Dict of dos data. Generally of the form, {label: {'energies':..,
            'densities': {'up':...}, 'efermi':efermi}}
        """
        return clean_json(self._doses)

    def get_plot(self, xlim=None, ylim=None):
        """
        Get a matplotlib plot showing the DOS.
        
        Args:
            xlim:
                Specifies the x-axis limits. Set to None for automatic 
                determination.
            ylim:
                Specifies the y-axis limits. 
        """
        from pymatgen.util.plotting_utils import get_publication_quality_plot
        plt = get_publication_quality_plot(12, 8)
        color_order = ['r', 'b', 'g', 'c']

        y = None
        alldensities = []
        allenergies = []
        """
        Note that this complicated processing of energies is to allow for
        stacked plots in matplotlib.
        """
        for key, dos in self._doses.items():
            energies = dos['energies']
            densities = dos['densities']
            if not y:
                y = {Spin.up: np.zeros(energies.shape), Spin.down: np.zeros(energies.shape)}
            newdens = {}
            for spin in [Spin.up, Spin.down]:
                if spin in densities:
                    if self.stack:
                        y[spin] += densities[spin]
                        newdens[spin] = y[spin].copy()
                    else:
                        newdens[spin] = densities[spin]
            allenergies.append(energies)
            alldensities.append(newdens)

        keys = list(self._doses.keys())
        keys.reverse()
        alldensities.reverse()
        allenergies.reverse()
        allpts = []
        for i, key in enumerate(keys):
            x = []
            y = []
            for spin in [Spin.up, Spin.down]:
                if spin in alldensities[i]:
                    densities = list(int(spin) * alldensities[i][spin])
                    energies = list(allenergies[i])
                    if spin == Spin.down:
                        energies.reverse()
                        densities.reverse()
                    x.extend(energies)
                    y.extend(densities)
            allpts.extend(zip(x, y))
            if self.stack:
                plt.fill(x, y, color=color_order[i % 4], label=str(key))
            else:
                plt.plot(x, y, color=color_order[i % 4], label=str(key))
            if not self.zero_at_efermi:
                ylim = plt.ylim()
                plt.plot([self._doses[key]['efermi'], self._doses[key]['efermi']], ylim, color_order[i % 4] + '--', linewidth=2)

        plt.xlabel('Energies (eV)')
        plt.ylabel('Density of states')
        if xlim:
            plt.xlim(xlim)
        if ylim:
            plt.ylim(ylim)
        else:
            xlim = plt.xlim()
            relevanty = [p[1] for p in allpts if p[0] > xlim[0] and p[0] < xlim[1]]
            plt.ylim((min(relevanty), max(relevanty)))

        if self.zero_at_efermi:
            ylim = plt.ylim()
            plt.plot([0, 0], ylim, 'k--', linewidth=2)

        plt.legend()
        leg = plt.gca().get_legend()
        ltext = leg.get_texts()  # all the text.Text instance in the legend
        plt.setp(ltext, fontsize=30)
        plt.tight_layout()
        return plt

    def save_plot(self, filename, img_format="eps", xlim=None, ylim=None):
        """
        Save matplotlib plot to a file.
        
        Args:
            filename:
                Filename to write to.
            img_format:
                Image format to use. Defaults to EPS.
            xlim:
                Specifies the x-axis limits. Set to None for automatic 
                determination.
            ylim:
                Specifies the y-axis limits. 
        """
        plt = self.get_plot(xlim, ylim)
        plt.savefig(filename, format=img_format)

    def show(self, xlim=None, ylim=None):
        """
        Show the plot using matplotlib.
        
        Args:
            xlim:
                Specifies the x-axis limits. Set to None for automatic 
                determination.
            ylim:
                Specifies the y-axis limits. 
        """
        plt = self.get_plot(xlim, ylim)
        plt.show()


class BSPlotter(object):
    """
    Class to plot or get data to facilitate the plot of band structure objects.
    """

    def __init__(self, bs):
        """
        Args:
            bs:
                A BandStructureSymmLine object.
        """
        self._bs = bs
        #Many ab initio codes do not give good results for the highest occupied bands, we therefore only
        #give 90% of the bands for plotting
        self._nb_bands = int(math.floor(self._bs._nb_bands * 0.9))

    def bs_plot_data(self, zero_to_efermi=True):

        """
        Get the data nicely formatted for a plot
        
        Args:
            zero_to_efermi:
                Automatically subtract off the Fermi energy from the eigenvalues
                and plot.
        
        Returns:
            A dict of the following format:
                ticks:
                    A dict with the 'distances' at which there is a kpoint (the
                    x axis) and the labels (None if no label)
                energy:
                    A dict storing bands for spin up and spin down data 
                    {Spin:[band_index][k_point_index]} as a list (one element
                    for each band) of energy for each kpoint.
                vbm:
                    A list of tuples (distance,energy) marking the vbms. The
                    energies are shifted with respect to the fermi level is the
                    option has been selected.
                cbm:
                    A list of tuples (distance,energy) marking the cbms. The
                    energies are shifted with respect to the fermi level is the
                    option has been selected.
                lattice:
                    The reciprocal lattice.
                zero_energy:
                    This is the energy used as zero for the plot.
                band_gap:
                    A string indicating the band gap and its nature (empty if
                    it's a metal).
                is_metal:
                    True if the band structure is metallic (i.e., there is at
                    least one band crossing the fermi level).
        """
        zero_energy = None

        if self._bs.is_metal():
            zero_energy = self._bs.efermi
        else:
            zero_energy = self._bs.get_vbm()['energy']

        if not zero_to_efermi:
            zero_energy = 0.0

        energy = {str(Spin.up): []}
        if self._bs.is_spin_polarized:
            energy = {str(Spin.up): [], str(Spin.down): []}
        distance = [self._bs._distance[j] for j in range(len(self._bs._kpoints))]
        ticks = self.get_ticks()
        for i in range(self._nb_bands):
            energy[str(Spin.up)].append([self._bs._bands[Spin.up][i][j] - zero_energy for j in range(len(self._bs._kpoints))])
        if self._bs.is_spin_polarized:
            for i in range(self._nb_bands):
                energy[str(Spin.down)].append([self._bs._bands[Spin.down][i][j] - zero_energy for j in range(len(self._bs._kpoints))])

        vbm = self._bs.get_vbm()
        cbm = self._bs.get_cbm()

        vbm_plot = []
        cbm_plot = []

        for index in cbm['kpoint_index']:
            cbm_plot.append((self._bs._distance[index],
                             cbm['energy'] - zero_energy if zero_to_efermi else cbm['energy']))

        for index in vbm['kpoint_index']:
            vbm_plot.append((self._bs._distance[index],
                             vbm['energy'] - zero_energy if zero_to_efermi else vbm['energy']))

        bg = self._bs.get_band_gap()
        direct = "Indirect"
        if bg['direct']:
            direct = "Direct"

        return {'ticks': ticks, 'distances': distance, 'energy': energy,
                'vbm':vbm_plot, 'cbm':cbm_plot,
                'lattice':self._bs._lattice_rec.to_dict,
                'zero_energy':zero_energy, 'is_metal':self._bs.is_metal(),
                'band_gap': "{} {} bandgap = {}".format(direct, bg['transition'], bg['energy']) if not self._bs.is_metal() else ""}

    def show(self, file_name=None, zero_to_efermi=True):
        """
        Show the bandstructure plot. Blue lines are up spin, red lines are down
        spin.
        
        Args:
            file_name:
                File name to write image to (e.g., plot.eps). If None, no image
                is created.
            zero_to_efermi:
                Automatically subtract off the Fermi energy from the eigenvalues
                and plot (E-Ef).
        """
        from pymatgen.util.plotting_utils import get_publication_quality_plot
        plt = get_publication_quality_plot(12, 8)
        from matplotlib import rc

        rc('text', usetex=True)

        #main internal config options
        e_min = -4
        e_max = 4
        if self._bs.is_metal():
            e_min = -10
            e_max = 10
        band_linewidth = 3

        #pylab.figure
        data = self.bs_plot_data(zero_to_efermi)
        for i in range(self._nb_bands):
                plt.plot(data['distances'],
                         [e for e in data['energy'][str(Spin.up)][i]],
                         'b-', linewidth=band_linewidth)
                if self._bs.is_spin_polarized:
                    plt.plot(data['distances'],
                             [e for e in data['energy'][str(Spin.down)][i]],
                             'r-', linewidth=band_linewidth)

        ticks = self.get_ticks()
        # ticks is dict wit keys: distances (array floats), labels (array str)
        logger.debug("ticks {t}".format(t=ticks))
        logger.debug("ticks has {n} distances and {m} labels".format(n=len(ticks['distance']), m=len(ticks['label'])))
        # Draw lines for BZ boundries
        for i in range(len(ticks['label'])):
            if ticks['label'][i] is not None:
                # don't print the same label twice
                if i != 0:
                    if (ticks['label'][i] == ticks['label'][i - 1]):
                        logger.debug("already print label... skipping label {i}".format(i=ticks['label'][i]))
                    else:
                        logger.debug("Adding a line at {d} for label {l}".format(d=ticks['distance'][i], l=ticks['label'][i]))
                        plt.axvline(ticks['distance'][i], color='k')
                else:
                    logger.debug("Adding a line at {d} for label {l}".format(d=ticks['distance'][i], l=ticks['label'][i]))
                    plt.axvline(ticks['distance'][i], color='k')

        #Sanitize only plot the uniq values
        uniq_d = []
        uniq_l = []
        temp_ticks = zip(ticks['distance'], ticks['label'])
        for i in xrange(len(temp_ticks)):
            if i == 0:
                uniq_d.append(temp_ticks[i][0])
                uniq_l.append(temp_ticks[i][1])
                logger.debug("Adding label {l} at {d}".format(l=temp_ticks[i][0], d=temp_ticks[i][1]))
            else:
                if temp_ticks[i][1] == temp_ticks[i - 1][1]:
                    logger.debug("Skipping label {i}".format(i=temp_ticks[i][1]))
                else:
                    logger.debug("Adding label {l} at {d}".format(l=temp_ticks[i][0], d=temp_ticks[i][1]))
                    uniq_d.append(temp_ticks[i][0])
                    uniq_l.append(temp_ticks[i][1])

        logger.debug("Unique labels are {i}".format(i=zip(uniq_d, uniq_l)))
        #pylab.gca().set_xticks(ticks['distance'])
        #pylab.gca().set_xticklabels(ticks['label'])
        plt.gca().set_xticks(uniq_d)
        plt.gca().set_xticklabels(uniq_l)

        #Main X and Y Labels
        plt.xlabel(r'$\mathrm{Wave\ Vector}$', fontsize=30)
        ylabel = r'$\mathrm{E\ -\ E_f\ (eV)}$' if zero_to_efermi else r'$\mathrm{Energy\ (eV)}$'
        plt.ylabel(ylabel, fontsize=30)

        # Draw Fermi energy, only if not the zero
        if not zero_to_efermi:
            ef = self._bs.efermi
            plt.axhline(ef, linewidth=2, color='k')

        # X range (K)
        #last distance point
        x_max = data['distances'][-1]
        plt.xlim(0, x_max)

        if self._bs.is_metal():
            # Plot A Metal
            if zero_to_efermi:
                plt.ylim(e_min, e_max)
            else:
                plt.ylim(self._bs.efermi + e_min, self._bs._efermi + e_max)
        else:

            for cbm in data['cbm']:
                plt.scatter(cbm[0], cbm[1], color='r', marker='o', s=100)

            for vbm in data['vbm']:
                plt.scatter(vbm[0], vbm[1], color='g', marker='o', s=100)

            plt.ylim(data['vbm'][0][1] + e_min, data['cbm'][0][1] + e_max)

        plt.tight_layout()
        if file_name is not None:
            plt.savefig(file_name)
            plt.close()
        else:
            plt.show()

    def get_ticks(self):
        """
        Get all ticks and labels for a band structure plot.
        
        Returns:
            A dict with 
                'distance': a list of distance at which ticks should be set.
                'label': a list of label for each of those ticks.
        """
        tick_distance = []
        tick_labels = []
        previous_label = self._bs._kpoints[0].label
        previous_branch = self._bs._branches[0]['name']
        for i, c in enumerate(self._bs._kpoints):
            if c.label != None:
                tick_distance.append(self._bs._distance[i])
                this_branch = None
                for b in self._bs._branches:
                    if i >= b['start_index'] and i <= b['end_index']:
                        this_branch = b['name']
                        break
                if c.label != previous_label and previous_branch != this_branch:
                    label1 = c.label
                    if label1.startswith("\\") or label1.find("_") != -1:
                        label1 = "$" + label1 + "$"
                    label0 = previous_label
                    if label0.startswith("\\") or label0.find("_") != -1:
                        label0 = "$" + label0 + "$"
                    tick_labels.pop()
                    tick_distance.pop()
                    tick_labels.append(label0 + "$|$" + label1)
                    #print label0+","+label1
                else:
                    if c.label.startswith("\\") or c.label.find("_") != -1:
                        tick_labels.append("$" + c.label + "$")
                    else:
                        tick_labels.append(c.label)
                previous_label = c.label
                previous_branch = this_branch
        return {'distance': tick_distance, 'label': tick_labels}

    def plot_compare(self, other_plotter):
        """
        plot two band structure for comparison. One is in red the other in blue
        (no difference in spins)
        TODO: still a lot of work to do that nicely!
        """
        import pylab
        data = self.bs_plot_data
        data_other = other_plotter.bs_plot_data
        for spin in data:
            for i in range(self._nb_bands):
                pylab.plot(data['distances'], data[spin][i], 'b-', linewidth=3)
        for spin in data_other:
            for i in range(self._nb_bands):
                pylab.plot(data['distances'], data_other[spin][i], 'r--', linewidth=3)


        ticks = self.get_ticks()

        pylab.gca().set_xticks(ticks['distance'])
        pylab.gca().set_xticklabels(ticks['label'])
        pylab.xlabel('Kpoints', fontsize='large')
        pylab.ylabel('Energy(eV)', fontsize='large')
        for i in range(len(ticks['label'])):
            if ticks['label'][i]:
                pylab.axvline(ticks['distance'][i], color='k')
        pylab.show()
        pylab.legend()

    def plot_brillouin(self):
        import pymatgen.command_line.qhull_caller
        import matplotlib as mpl
        import matplotlib.pyplot as plt
        mpl.rcParams['legend.fontsize'] = 10

        fig = plt.figure()
        ax = fig.gca(projection='3d')
        vec1 = self._bs._lattice_rec.matrix[0]
        vec2 = self._bs._lattice_rec.matrix[1]
        vec3 = self._bs._lattice_rec.matrix[2]

        #make the grid
        max_x = -1000
        max_y = -1000
        max_z = -1000
        min_x = 1000
        min_y = 1000
        min_z = 1000
        list_k_points = []
        for i in [-1, 0, 1]:
            for j in [-1, 0, 1]:
                for k in [-1, 0, 1]:
                    list_k_points.append(i * vec1 + j * vec2 + k * vec3)
                    if list_k_points[-1][0] > max_x:
                        max_x = list_k_points[-1][0]
                    if list_k_points[-1][1] > max_y:
                        max_y = list_k_points[-1][1]
                    if list_k_points[-1][2] > max_z:
                        max_z = list_k_points[-1][0]

                    if list_k_points[-1][0] < min_x:
                        min_x = list_k_points[-1][0]
                    if list_k_points[-1][1] < min_y:
                        min_y = list_k_points[-1][1]
                    if list_k_points[-1][2] < min_z:
                        min_z = list_k_points[-1][0]

        vertex = pymatgen.command_line.qhull_caller.qvertex_target(list_k_points, 13)
        lines = pymatgen.command_line.qhull_caller.get_lines_voronoi(vertex)

        for i in range(len(lines)):
            vertex1 = lines[i]['start']
            vertex2 = lines[i]['end']
            ax.plot([vertex1[0], vertex2[0]], [vertex1[1], vertex2[1]], [vertex1[2], vertex2[2]], color='k')

        for b in self._bs._branches:
            vertex1 = self._bs._kpoints[b['start_index']].cart_coords
            vertex2 = self._bs._kpoints[b['end_index']].cart_coords
            ax.plot([vertex1[0], vertex2[0]], [vertex1[1], vertex2[1]], [vertex1[2], vertex2[2]], color='r', linewidth=3)

        for k in self._bs._kpoints:
            if k.label:
                label = k.label
                if k.label.startswith("\\") or k.label.find("_") != -1:
                    label = "$" + k.label + "$"
                off = 0.01
                ax.text(k.cart_coords[0] + off, k.cart_coords[1] + off, k.cart_coords[2] + off, label, color='b', size='25')
                ax.scatter([k.cart_coords[0]], [k.cart_coords[1]], [k.cart_coords[2]], color='b')

        # make ticklabels and ticklines invisible
        for a in ax.w_xaxis.get_ticklines() + ax.w_xaxis.get_ticklabels():
            a.set_visible(False)
        for a in ax.w_yaxis.get_ticklines() + ax.w_yaxis.get_ticklabels():
            a.set_visible(False)
        for a in ax.w_zaxis.get_ticklines() + ax.w_zaxis.get_ticklabels():
            a.set_visible(False)

        ax.grid(False)

        plt.show()
        ax.axis("off")




#!/usr/bin/env python

'''
This module implements more advanced transformations.
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "Jul 24, 2012"

import numpy as np

from pymatgen.core.periodic_table import smart_element_or_specie
from pymatgen.transformations.transformation_abc import AbstractTransformation
from pymatgen.transformations.standard_transformations import SubstitutionTransformation, OrderDisorderedStructureTransformation
from pymatgen.command_line.enumlib_caller import EnumlibAdaptor
from pymatgen.analysis.ewald import EwaldSummation
from pymatgen.core.structure_modifier import SupercellMaker


class ChargeBalanceTransformation(AbstractTransformation):
    """
    This is a transformation that disorders a structure to make it charge
    balanced, given an oxidation state-decorated structure.
    """

    def __init__(self, charge_balance_sp):
        '''
        Args:
            charge_balance_sp
                specie to add or remove. Currently only removal is supported
        '''
        self._charge_balance_sp = str(charge_balance_sp)

    def apply_transformation(self, structure):
        charge = structure.charge
        specie = smart_element_or_specie(self._charge_balance_sp)
        num_to_remove = charge / specie.oxi_state
        num_in_structure = structure.composition[specie]
        removal_fraction = num_to_remove / num_in_structure
        if removal_fraction < 0:
            raise ValueError('addition of specie not yet supported by '
                             'ChargeBalanceTransformation')
        mapping = {self._charge_balance_sp : {self._charge_balance_sp : 1 - removal_fraction}}
        return SubstitutionTransformation(mapping).apply_transformation(structure)

    def __str__(self):
        return "Charge Balance Transformation : Species to remove = {}".format(str(self._charge_balance_sp))

    def __repr__(self):
        return self.__str__()

    @property
    def inverse(self):
        return None

    @property
    def is_one_to_many(self):
        return False

    @property
    def to_dict(self):
        d = {'name' : self.__class__.__name__, 'version': __version__ }
        d['init_args'] = {'charge_balance_sp' : self._charge_balance_sp}
        d['module'] = self.__class__.__module__
        d['class'] = self.__class__.__name__
        return d


class SuperTransformation(AbstractTransformation):
    '''
    This is a transformation that is inherently one-to-many. It is constructed
    from a list of transformations and returns one structure for each
    transformation. The primary use for this class is extending a transmuter
    object.
    '''

    def __init__(self, transformations):
        '''
        Args:
            transformations:
                list of transformations to apply to a structure. One
                transformation is applied to each output structure.
        '''
        self._transformations = transformations

    def apply_transformation(self, structure, return_ranked_list=False):
        if not return_ranked_list:
            raise ValueError('SuperTransformation has no single best structure '
                             'output. Must use return_ranked_list')
        structures = []
        for t in self._transformations:
            structures.append({'transformation': t,
                               'structure': t.apply_transformation(structure)})

        return structures

    def __str__(self):
        return "Super Transformation : Transformations = {}".format(' '.join([str(t) for t in self._transformations]))

    def __repr__(self):
        return self.__str__()

    @property
    def inverse(self):
        return None

    @property
    def is_one_to_many(self):
        return True

    @property
    def to_dict(self):
        d = {'name' : self.__class__.__name__, 'version': __version__ }
        d['init_args'] = {'transformations' : self._transformations}
        d['module'] = self.__class__.__module__
        d['class'] = self.__class__.__name__
        return d


class MultipleSubstitutionTransformation(object):
    '''
    Performs multiple substitutions on a structure. For example, can do a
    fractional replacement of Ge in LiGePS with a list of species, creating one
    structure for each substitution. Ordering is done using a dummy element so
    only one ordering must be done per substitution oxidation state. Charge
    balancing of the structure is optionally performed.
    
    .. note::
        There are no checks to make sure that removal fractions are possible
        and rounding may occur. Currently charge balancing only works for
        removal of species.
    '''

    def __init__(self, sp_to_replace, r_fraction, substitution_dict,
                 charge_balance_species=None, order=True):
        '''
        Performs multiple fractional substitutions on a transmuter.
        
        Args:
            sp_to_replace
                species to be replaced
            r_fraction
                fraction of that specie to replace
            substitution_dict
                dictionary of the format
                {2: ["Mg", "Ti", "V", "As", "Cr", "Ta", "N", "Nb"], 
                3: ["Ru", "Fe", "Co", "Ce", "As", "Cr", "Ta", "N", "Nb"],
                4: ["Ru", "V", "Cr", "Ta", "N", "Nb"], 
                5: ["Ru", "W", "Mn"]
                }
                The number is the charge used for each of the list of elements
                (an element can be present in multiple lists)
            charge_balance_species:
                If specified, will balance the charge on the structure using 
                that specie.
        '''
        self._sp_to_replace = sp_to_replace
        self._r_fraction = r_fraction
        self._substitution_dict = substitution_dict
        self._charge_balance_species = charge_balance_species
        self._order = order

    def apply_transformation(self, structure, return_ranked_list=False):
        if not return_ranked_list:
            raise ValueError('MultipleSubstitutionTransformation has no single '
                             'best structure output. Must use '
                             'return_ranked_list.')
        outputs = []
        for charge, el_list in self._substitution_dict.items():
            mapping = {}
            if charge > 0:
                sign = '+'
            else:
                sign = '-'
            mapping[self._sp_to_replace] = {self._sp_to_replace:1 - self._r_fraction,
                            'X{}{}'.format(str(charge), sign):self._r_fraction}
            trans = SubstitutionTransformation(mapping)
            dummy_structure = trans.apply_transformation(structure)
            if self._charge_balance_species is not None:
                cbt = ChargeBalanceTransformation(self._charge_balance_species)
                dummy_structure = cbt.apply_transformation(dummy_structure)
            if self._order:
                trans = OrderDisorderedStructureTransformation()
                dummy_structure = trans.apply_transformation(dummy_structure)

            for el in el_list:
                if charge > 0:
                    sign = '+'
                else:
                    sign = '-'
                st = SubstitutionTransformation({'X{}+'.format(str(charge)): '{}{}{}'.format(el, charge, sign)})
                new_structure = st.apply_transformation(dummy_structure)
                outputs.append({'structure' : new_structure})
        return outputs

    def __str__(self):
        return "Multiple Substitution Transformation : Substitution on " + \
               "{}".format(self._sp_to_replace)

    def __repr__(self):
        return self.__str__()

    @property
    def inverse(self):
        return None

    @property
    def is_one_to_many(self):
        return True

    @property
    def to_dict(self):
        d = {'name' : self.__class__.__name__, 'version': __version__ }
        d['init_args'] = {'sp_to_replace' : self._sp_to_replace,
                          'r_fraction' : self._r_fraction,
                          'substitution_dict' : self._substitution_dict,
                        'charge_balance_species' : self._charge_balance_species}
        d['module'] = self.__class__.__module__
        d['class'] = self.__class__.__name__
        return d


class EnumerateStructureTransformation(AbstractTransformation):
    """
    Order a disordered structure using enumlib. For complete orderings, this
    generally produces fewer structures that the OrderDisorderedStructure
    transformation, and at a much faster speed.
    
    .. note:: Early alpha, and not completely tested.
    """

    def __init__(self, min_cell_size=1, max_cell_size=1, symm_prec=0.1):
        '''
        Args:
            min_cell_size:
                The minimum cell size wanted. Must be an int. Defaults to 1.
            max_cell_size:
                The maximum cell size wanted. Must be an int. Defaults to 1.
            symm_prec:
                Tolerance to use for symmetry.
        '''
        self.symm_prec = symm_prec
        self.min_cell_size = min_cell_size
        self.max_cell_size = max_cell_size

    def apply_transformation(self, structure, return_ranked_list=False):
        """
        Return either a single ordered structure or a sequence of all ordered
        structures.
        
        Args:
            structure:
                Structure to order. 
            return_ranked_list:
                Boolean stating whether or not multiple structures are
                returned. If return_ranked_list is a number, that number of
                structures is returned.
                
        Returns:
            Depending on returned_ranked list, either a transformed structure 
            or a list of dictionaries, where each dictionary is of the form 
            {'structure' = .... , 'other_arguments'}
            
            The list of ordered structures is ranked by ewald energy / atom, if
            the input structure is an oxidation state decorated structure.
            Otherwise, it is ranked by number of sites, with smallest number of
            sites first.
        """
        try:
            num_to_return = int(return_ranked_list)
        except:
            num_to_return = 1

        if structure.is_ordered:
            raise ValueError("Enumeration can be carried out only on "
                             "disordered structures!")

        contains_oxidation_state = True
        for sp in structure.composition.elements:
            if not hasattr(sp, "oxi_state"):
                contains_oxidation_state = False
                break

        adaptor = EnumlibAdaptor(structure, min_cell_size=self.min_cell_size,
                                 max_cell_size=self.max_cell_size,
                                 symm_prec=self.symm_prec)

        adaptor.run()
        structures = adaptor.structures
        original_latt = structure.lattice
        inv_latt = np.linalg.inv(original_latt.matrix)
        ewald_matrices = {}
        all_structures = []
        for s in structures:
            new_latt = s.lattice
            transformation = np.dot(new_latt.matrix, inv_latt)
            transformation = tuple([tuple([int(round(cell)) for cell in row])\
                                    for row in transformation])
            if contains_oxidation_state:
                if transformation not in ewald_matrices:
                    maker = SupercellMaker(structure, transformation)
                    ewald = EwaldSummation(maker.modified_structure)
                    ewald_matrices[transformation] = ewald
                else:
                    ewald = ewald_matrices[transformation]
                energy = ewald.compute_sub_structure(s)
                all_structures.append({'num_sites':len(s), 'energy':energy,
                                       'structure':s})
            else:
                all_structures.append({'num_sites':len(s), 'structure':s})

        def sort_func(s):
            return s['energy'] / s['num_sites'] if contains_oxidation_state \
                else s['num_sites']

        self._all_structures = sorted(all_structures, key=sort_func)

        if return_ranked_list:
            return self._all_structures[0:num_to_return]
        else:
            return self._all_structures[0]['structure']

    def __str__(self):
        return "EnumerateStructureTransformation"

    def __repr__(self):
        return self.__str__()

    @property
    def inverse(self):
        return None

    @property
    def is_one_to_many(self):
        return True

    @property
    def to_dict(self):
        d = {'name' : self.__class__.__name__, 'version': __version__}
        d['init_args'] = {'symm_prec' : self.symm_prec,
                          'min_cell_size': self.min_cell_size,
                          'max_cell_size': self.max_cell_size}
        d['module'] = self.__class__.__module__
        d['class'] = self.__class__.__name__
        return d
